/*
 * file SimpleCreateQuery.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.SimpleCreateQuery
 *
 *  Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.stp.client.samples;

import static javax.swing.JOptionPane.showInputDialog;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;
import javax.wvcm.ProviderFactory;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyRequestItem.PropertyRequest;

import com.ibm.rational.wvcm.stp.StpLocation;
import com.ibm.rational.wvcm.stp.StpProvider;
import com.ibm.rational.wvcm.stp.StpResource;
import com.ibm.rational.wvcm.stp.StpProvider.Domain;
import com.ibm.rational.wvcm.stp.cc.CcProvider;
import com.ibm.rational.wvcm.stp.cq.CqFieldDefinition;
import com.ibm.rational.wvcm.stp.cq.CqProvider;
import com.ibm.rational.wvcm.stp.cq.CqQuery;
import com.ibm.rational.wvcm.stp.cq.CqRecordType;
import com.ibm.rational.wvcm.stp.cq.CqResultSet;
import com.ibm.rational.wvcm.stp.cq.CqQuery.DisplayField;
import com.ibm.rational.wvcm.stp.cq.CqQuery.Filter;
import com.ibm.rational.wvcm.stp.cq.CqQuery.Filter.Operation;
import com.ibm.rational.wvcm.stp.cq.CqQuery.FilterLeaf.TargetType;

/**
 * @author trw
 * 
 */
public class SimpleCreateQuery
{

    public static void main(String[] args) throws Exception
    {
        g_provider = (CqProvider) ProviderFactory
                .createProvider(CqProvider.CQ_ONLY_PROVIDER_CLASS, g_callback);

        // Get the location for the new query from the user
        StpLocation location = g_provider.stpLocation
                    (showInputDialog("Where do you want to create the query?",
                                     "cq.query:<query path>@<dbset>/<userdb>"));
        
        // Get a proxy for the record type the query will target
        CqRecordType rt = g_provider.buildProxy(CqRecordType.class, 
                                                "cq.record:Defect@" 
                                                    + location.getRepo());
        
        // Construct display field array and filter expression for the query
        DisplayField[] fields =
            new DisplayField[] {
                buildColumn(rt, "id"), 
                buildColumn(rt, "headline"),
                buildColumn(rt, "customer.name"),
                buildColumn(rt, "project.name"), 
                buildColumn(rt, "description") };
        Filter filter =
            g_provider.buildFilterLeaf(buildPath(rt, "Owner.login_name"),
                                       Operation.IS_EQUAL,
                                       TargetType.CONSTANT,
                                       "engineer"); 
        
        // Create a permanent version of this query
        CqQuery query = g_provider.cqQuery(location);

        query.setPrimaryRecordType(rt);
        query.setDisplayFields(fields);
        query.setFiltering(filter);
        
        query.doCreateQuery(null);

        // Execute the query and print the results (optional)
        CqResultSet results = query.doExecute(1, Integer.MAX_VALUE, null);
    
        while (results.hasNext()) {
            String[] row = results.next().getStrings();
            System.out.println(row[0] + ": (from " + row[2] + ") " + row[1]);
        }
    
        results.release();
        
        // Clean up (present only for a repeatable demonstration)
        query.doUnbindAll(null);
    }

    static DisplayField buildColumn(CqRecordType rt,
                                       String path) throws WvcmException
    {
        return rt.cqProvider().buildDisplayField(buildPath(rt, path), true);
    }

    static CqFieldDefinition[] buildPath(CqRecordType rt,
                                            String path) throws WvcmException
    {
        List<CqFieldDefinition> result = new ArrayList<CqFieldDefinition>();
        if (rt.lookupProperty(CqRecordType.FIELD_DEFINITIONS) instanceof Exception)
            rt = (CqRecordType) rt.doReadProperties(QUERY_RECORD_TYPE_PROPS);
        for (String name : path.split("\\.")) {
            if (rt == null)
                throw new IllegalArgumentException("Field preceeding " + name
                    + " does not reference a record");
            CqFieldDefinition def = find(rt, name);
            result.add(def);
            rt = null;
            switch (def.getFieldType()) {
            case RESOURCE:
            case RESOURCE_LIST:
                rt =
                    (CqRecordType) def.getReferencedRecordType()
                        .doReadProperties(QUERY_RECORD_TYPE_PROPS);
            }
        }
        return result.toArray(new CqFieldDefinition[result.size()]);
    }

    static CqFieldDefinition find(CqRecordType rt,
                                   String name) throws WvcmException
    {
        for (CqFieldDefinition fieldDef : rt.getFieldDefinitions())
            if (fieldDef.getDisplayName().compareToIgnoreCase(name) == 0)
                return fieldDef;
        throw new IllegalArgumentException("'" + name + "' is not a field ");
    }

    private static final PropertyRequest FIELD_DEFN_PROPS = new PropertyRequest(StpResource.DISPLAY_NAME, CqFieldDefinition.VALUE_TYPE, CqFieldDefinition.FIELD_TYPE, CqFieldDefinition.REFERENCED_RECORD_TYPE);

    private static final PropertyRequest QUERY_RECORD_TYPE_PROPS = new PropertyRequest(StpResource.DISPLAY_NAME, CqRecordType.FIELD_DEFINITIONS.nest(FIELD_DEFN_PROPS));

    static StpProvider.StpCallback g_callback = new StpProvider.StpCallback()
    {
        public Authentication getAuthenticationEx(Domain domain,
                                                  String realm,
                                                  int retryCount,
                                                  StpProvider provider,
                                                  WvcmException failure) throws WvcmException
        {
            if (m_unpw != null && retryCount == 0)
                return m_unpw;
            String title = "Enter " + domain + " Username '+' Password for " + realm + " [" + retryCount + "]";
            if (failure != null)
                title = "Login failed: " + failure + "\n" + title;
            String unpw = JOptionPane.showInputDialog(title, "admin+");
            if (unpw == null || unpw.length() == 0)
                throw new IllegalAccessError("User canceled request");
            return m_unpw = unpw.equals("anonymous")? null : new UnPw(unpw);
        }

        public Authentication getAuthentication(String r,
                                                int c)
        {
            return null;
        }

        private UnPw m_unpw;
    };

    /**
     * A simple Authentication object in which the username and password
     * obtained from the user is cached for use by the CM API.
     */
    static class UnPw implements CcProvider.CcAuthentication {
        /**
         * Constructs an Authentication object
         * 
         * @param unpw A String containing username+password.
         */
        UnPw(String unpw) {  m_data = unpw.split("\\+", -2); }

        public String loginName() { return m_data[0]; }
        public String password() { return m_data.length > 1 ? m_data[1] : ""; };
        public String getPrimaryGroupName() {  return m_data.length > 2 ? m_data[2] : ""; }

        /* (non-Javadoc)
         * @see com.ibm.rational.wvcm.stp.cc.CcProvider.CcAuthentication#getGroupList()
         */
        public List<String> getGroupList()
        {
            return null;
        }

        /** The cached credentials */
        private String[] m_data;

    }

    static CqProvider g_provider;
}
