/*
 * file CheckoutDemo.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.CheckoutDemo
 *
 *  Copyright IBM Corporation 2007, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.stp.client.samples.cc;

import javax.wvcm.PropertyRequestItem.PropertyRequest;

import com.ibm.rational.wvcm.stp.cc.CcFile;

/**
 * Checkout a file in a web view.
 * Demonstrates basic web view operations.
 */
public class CheckoutDemo extends CcDemoBase {

    public void testSimpleCheckout() throws Exception {

        ViewHelper viewHelper = ViewHelper.getNonUcmViewHelper(this);

        // Create a version controlled file we can work with
        CcFile demoFile = viewHelper.createDemoFile("CheckoutDemo.txt", true/*version controlled*/);

        // Fetch the demo file.
        // This file should already exist, should be loaded in this web view,
        // should be version controlled, and should be checked in.
        // Verify that by reading the file's properties.
        trace("Got demo file: " + demoFile.clientResourceFile());
                
        PropertyRequest wantedProps = new PropertyRequest(
                CcFile.LOAD_STATE,
                CcFile.IS_VERSION_CONTROLLED,
                CcFile.IS_CHECKED_OUT);

        demoFile = (CcFile) demoFile.doReadProperties(wantedProps);
        assertEquals(CcFile.LoadState.LOADED, demoFile.getLoadState());
        assertTrue(demoFile.getIsVersionControlled());
        assertFalse(demoFile.getIsCheckedOut());

        // Check out the demo file.
        // As part of the checkout operation, ask for the file's checkout-related
        // properties so we can verify that the operation succeeded.
        trace("Checking out demo file: " + demoFile.clientResourceFile());
        demoFile = (CcFile) demoFile.doCheckout(null, wantedProps);
        assertTrue(demoFile.getIsCheckedOut());
    }
}
